#!/usr/bin/env python3

# Reference to lecture:
# https://teaching.leonardotamiano.xyz/cns/labs/lab_04/

from Crypto.PublicKey import RSA
from Crypto.Util.number import bytes_to_long, long_to_bytes

BIT_SIZE = 1024

def generate_key(bits):
    KEY = RSA.generate(BIT_SIZE)
    return KEY

def dump_key(rsa_key):
    print("Private parameters")
    print(f"D={rsa_key.d}")
    print(f"P={rsa_key.p}")
    print(f"Q={rsa_key.q}")
    print("Public parameters")
    print(f"N={rsa_key.n}")
    print(f"E={rsa_key.e}")
    print("==========================")

def encryption_example(key, plaintext):
    print(f"About to encrypt a new message")
    print(f"Plaintext:\n{plaintext}")
    ciphertext = pow(plaintext, key.e, key.n)
    print(f"Ciphertext:\n{ciphertext}")
    print("==========================")
    return ciphertext

def decryption_example(key, ciphertext):
    print(f"About to decrypt a new message")
    print(f"Ciphertext:\n{ciphertext}")
    plaintext = pow(ciphertext, key.d, key.n)
    print(f"Plaintext:\n{plaintext}")    
    print("==========================")
    return plaintext

def encryption_decryption_test():
    key = generate_key(BIT_SIZE)
    dump_key(key)
    m = b"The tutor of CNS sucks at teaching"
    p = bytes_to_long(m)
    c = encryption_example(key, p)
    p2 = decryption_example(key, c)
    assert p == p2, "Oops, we broke math"
    
if __name__ == "__main__":
    encryption_decryption_test()    
