#!/usr/bin/env python3

# Reference to lecture:
# https://teaching.leonardotamiano.xyz/cns/labs/lab_04/

import codecs
from hashlib import sha256
from Crypto.PublicKey import RSA
from Crypto.Util.number import bytes_to_long, long_to_bytes

BIT_SIZE = 1024

def compute_signature(msg, key):
    print(f"Computing signature for: {msg}")
    hash_value = sha256(msg.encode('utf-8'))
    bytes_value = codecs.decode(hash_value.hexdigest(), 'hex_codec')
    hash_number = bytes_to_long(bytes_value) % key.n
    signature_value = pow(hash_number, key.d, key.n)
    print(f"Signature computed: {signature_value}")
    return (msg, signature_value)

def verify_signature(signature, key):
    (msg, sig_value) = signature
    
    hash_value = sha256(msg.encode('utf-8'))
    bytes_value = codecs.decode(hash_value.hexdigest(), 'hex_codec')
    hash_number = bytes_to_long(bytes_value) % key.n    

    signature_check = pow(sig_value, key.e, key.n)

    if hash_number == signature_check:
        print("OK: Signature correctly verified!")
    else:
        print("NOPE: Signature failed!")

def main():
    global BIT_SIZE
    key = RSA.generate(BIT_SIZE)
    msg = "Hello World!"
    signature = compute_signature(msg, key)
    verify_signature(signature, key)    
    
if __name__ == "__main__":
    main()
